/*
 * File:   main.c
 * Author: david
 *
 * Created on May 15, 2021, 1:50 PM
 */


// PIC16F628A Configuration Bit Settings

// 'C' source line config statements

// CONFIG
#pragma config FOSC = INTOSCCLK // Oscillator Selection bits (INTOSC oscillator: CLKOUT function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF        // Watchdog Timer Enable bit (WDT enabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = OFF      // Brown-out Detect Enable bit (BOD disabled)
#pragma config LVP = ON         // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>

// Port definitions
#define LINEFREQ        RB0     // RB0 is 60Hz Line Frequency input
#define LINEFREQ_MASK   0x01
#define PUMP            RB1     // RB1 is Pump control output
#define PUMP_MASK       0x02
#define TURBINE         RB2     // RB2 is Turbine control output
#define TURBINE_MASK    0x04
#define HEARTBEAT       RB3     // RB3 is Heartbeat output
#define HEARTBEAT_MASK  0x08
#define THERMOSTAT      RB5     // RB5 is thermostat contacts input
#define THERMOSTAT_MASK 0x20
#define PORTB_OUTPUTS   (PUMP_MASK | TURBINE_MASK | HEARTBEAT_MASK)

// State machine definitions
#define STATE_OFF       0
#define STATE_WET_PADS  1
#define STATE_OPERATE   2
#define STATE_DRY_PADS  3

void main(void)
{
    int linefreq;
    int linefreq_prev;
    int thermostat;
    int count_60hz;
    int count_seconds;
    int doit;
    int state;
    
    OPTION_REGbits.nRBPU = 1;
    TRISB = ~PORTB_OUTPUTS;
    PORTB = 0;
    
    count_60hz = 0;
    state = STATE_OFF;
    while(1) {
        // Capture the rising edge of the line frequency input;
        // i.e., when it is true but before it was false
        linefreq = PORTBbits.LINEFREQ;
        doit = linefreq && !linefreq_prev;
        linefreq_prev = linefreq;
        if(!doit)
            continue;
        // Actions upon edge.  Advance the 1/60sec counter
        count_60hz++;
        if(count_60hz == 30) {
            PORTBbits.HEARTBEAT = 0;
            continue;
        }
        // Main action upon second intervals
        if(count_60hz == 60) {
            count_60hz = 0;
            PORTBbits.HEARTBEAT = 1;
            
            thermostat = PORTBbits.THERMOSTAT;
            switch(state) {
                 
                case STATE_OFF:
                    // Stay off until thermostat says True.
                    // Note thermostat is in Heat Mode, so
                    // contacts are closed and signal is grounded
                    // if temperature is below setpoint.
                    // Therefore cooler activated if contact opens
                    // and signal goes high.
                    if(thermostat) {
                        PORTBbits.PUMP = 1;
                        count_seconds = 0;
                        state = STATE_WET_PADS;
                    }
                    break;
                    
                case STATE_WET_PADS:
                    // Abort if thermostat closes again
                    if(!thermostat) {
                        PORTBbits.PUMP = 0;
                        state = STATE_OFF;
                        break;
                    }
                    count_seconds++;
                    if(count_seconds == 30) {
                        PORTBbits.TURBINE = 1;
                        state = STATE_OPERATE;
                    }
                    break;
                    
                case STATE_OPERATE:
                    if(!thermostat) {
                        PORTBbits.PUMP = 0;
                        state = STATE_DRY_PADS;
                        count_seconds = 0;
                        break;
                    }
                    break;
                    
                case STATE_DRY_PADS:
                    // Abort if thermostat opens again
                    if(thermostat) {
                        PORTBbits.PUMP = 1;
                        state = STATE_OPERATE;
                        break;
                    }
                    count_seconds++;
                    if(count_seconds == 30) {
                        PORTBbits.TURBINE = 0;
                        state = STATE_OFF;
                    }
                    break;
            }
        }
    }
}
